"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = promiseFromProcess;
function _commander() {
  const data = _interopRequireDefault(require("commander"));
  _commander = function () {
    return data;
  };
  return data;
}
var _package = require("../package.json");
function _simpleGit() {
  const data = _interopRequireDefault(require("simple-git"));
  _simpleGit = function () {
    return data;
  };
  return data;
}
function _fs() {
  const data = _interopRequireDefault(require("fs"));
  _fs = function () {
    return data;
  };
  return data;
}
function _path() {
  const data = _interopRequireDefault(require("path"));
  _path = function () {
    return data;
  };
  return data;
}
function _ncp2() {
  const data = _interopRequireDefault(require("ncp"));
  _ncp2 = function () {
    return data;
  };
  return data;
}
function _util() {
  const data = require("util");
  _util = function () {
    return data;
  };
  return data;
}
function _commandExists() {
  const data = _interopRequireDefault(require("command-exists"));
  _commandExists = function () {
    return data;
  };
  return data;
}
function _crossSpawn() {
  const data = _interopRequireDefault(require("cross-spawn"));
  _crossSpawn = function () {
    return data;
  };
  return data;
}
function _rimraf() {
  const data = _interopRequireDefault(require("rimraf"));
  _rimraf = function () {
    return data;
  };
  return data;
}
function _tempy() {
  const data = _interopRequireDefault(require("tempy"));
  _tempy = function () {
    return data;
  };
  return data;
}
function _chalk() {
  const data = _interopRequireDefault(require("chalk"));
  _chalk = function () {
    return data;
  };
  return data;
}
var emoji = _interopRequireWildcard(require("./emoji"));
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
// flowlint-next-line untyped-import:off
// flowlint-next-line untyped-import:off
// flowlint-next-line untyped-import:off
const TEMPLATES_DIR = _path().default.resolve(__dirname, '../templates');
const ncp = (0, _util().promisify)(_ncp2().default);
// eslint-disable-next-line no-console
const log = console.log;
_commander().default.name('create-react-app').version(_package.version).arguments('<path-to-new-app>').action(command => {
  run(command).catch(reason => {
    // eslint-disable-next-line no-console
    console.error((0, _chalk().default)`${emoji.error} {red ${reason.message}}`);
    process.exit(1);
  });
}).parse();
async function run(packagePath) {
  log((0, _chalk().default)`${emoji.progress} {green Creating Parcel app at}`, _chalk().default.bold.underline(packagePath));
  if (await fsExists(packagePath)) {
    throw new Error(`File or directory at ${packagePath} already exists`);
  }
  let tempPath = _tempy().default.directory();
  try {
    await createApp(_path().default.basename(packagePath), tempPath);
  } catch (e) {
    await (0, _rimraf().default)(tempPath);
    throw e;
  }
  await _fs().default.promises.rename(tempPath, packagePath);
  log((0, _chalk().default)`{green ${emoji.success} Successfully created a new Parcel app at {bold.underline ${packagePath}}.}`);
  log((0, _chalk().default)`${emoji.info}  {dim Run} {bold cd ${packagePath}} {dim and then} {bold ${usesYarn ? 'yarn' : 'npm run'} start} {dim to start developing with Parcel.}`);
}
async function createApp(packageName, tempPath) {
  log(emoji.progress, 'Creating package directory...');
  const git = (0, _simpleGit().default)({
    baseDir: tempPath
  });
  log(emoji.progress, 'Initializing git repository...');
  await git.init();
  log(emoji.progress, 'Adding templates...');
  await Promise.all([async function writePackageJson() {
    const packageJson = JSON.parse(await _fs().default.promises.readFile(_path().default.join(TEMPLATES_DIR, 'package.json'), 'utf8'));
    await _fs().default.promises.writeFile(_path().default.join(tempPath, 'package.json'), JSON.stringify({
      name: packageName,
      ...packageJson
    }, null, 2));
  }(), ncp(_path().default.join(TEMPLATES_DIR, 'default'), tempPath)]);
  log(emoji.progress, 'Installing packages...');
  await installPackages(['parcel@nightly', 'postcss', '@babel/core'], {
    cwd: tempPath,
    isDevDependency: true
  });
  await installPackages(['react', 'react-dom'], {
    cwd: tempPath
  });
  log(_chalk().default.green(emoji.progress, 'Creating initial commit...'));
  await git.add('.');
  await git.commit('Initial commit created with @parcel/create-react-app');
}
async function fsExists(filePath) {
  try {
    return (await _fs().default.promises.stat(filePath)) && true;
  } catch {
    return false;
  }
}
let usesYarn;
async function installPackages(packageExpressions, opts) {
  log(emoji.progress, (0, _chalk().default)`{dim Installing}`, _chalk().default.bold(...packageExpressions));
  if (usesYarn == null) {
    usesYarn = await (0, _commandExists().default)('yarn');
    if (!usesYarn && !(await (0, _commandExists().default)('npm'))) {
      throw new Error('Neither npm nor yarn found on system');
    }
  }
  if (usesYarn) {
    return promiseFromProcess((0, _crossSpawn().default)('yarn', ['add', opts.isDevDependency ? '--dev' : null, ...packageExpressions].filter(Boolean), {
      cwd: opts.cwd
    }));
  }
  return promiseFromProcess((0, _crossSpawn().default)('npm', ['install', opts.isDevDependency ? '--save-dev' : null, ...packageExpressions].filter(Boolean), {
    cwd: opts.cwd
  }));
}
function promiseFromProcess(childProcess) {
  return new Promise((resolve, reject) => {
    childProcess.on('error', reject);
    childProcess.on('close', code => {
      if (code !== 0) {
        reject(new Error('Child process failed'));
        return;
      }
      resolve();
    });
  });
}